import java.io.File;
import java.io.OutputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.xml.sax.XMLReader;

import org.apache.fop.apps.Driver;
import org.apache.fop.apps.XSLTInputHandler;

/**
 * Example servlet to generate a PDF from an XSL-FO document
 */
public class EzFopServlet extends HttpServlet {

    public void doGet(HttpServletRequest request, HttpServletResponse response)
    	throws ServletException, IOException {

		try {
			// get the application root for this web app
			String appRoot  = getServletContext().getRealPath("/");

			String xmlFileName = "booklist.xml";
			String xslFileName = "booklist_table.xsl";
			File xmlFile = new File(appRoot + xmlFileName);
			File xslFile = new File(appRoot + xslFileName);

			// diagnostic messages
			System.out.println("EzFopServlet:  XSL-FO formatting:");
			System.out.println("xml = " + xmlFile);
			System.out.println("xsl = " + xslFile + "\n\n");

			// setup the driver for PDF
			Driver driver = new Driver();
			driver.setRenderer(Driver.RENDER_PDF);

			// create an input handler for the XSLT transformation
			XSLTInputHandler inputHandler = new XSLTInputHandler(xmlFile, xslFile);
			XMLReader parser = inputHandler.getParser();

			// setup the output for XSL-FO formatter process
			// temporarily place in a ByteArrayOutputStream
			ByteArrayOutputStream out = new ByteArrayOutputStream();
			driver.setOutputStream(out);

			// Run the formatter based on the XSL-FO document
			driver.render(parser, inputHandler.getInputSource());

			// The out object has the result of the XSL-FO formatter process
			//  Retrieve the content from ByteArray
			byte[] content = out.toByteArray();

			// Setup the response for the web browser
			response.setContentType("application/pdf");
			response.setContentLength(content.length);

			// Finally, send the result to the browser!
			OutputStream outputToBrowser = response.getOutputStream();
			outputToBrowser.write(content);
			outputToBrowser.flush();
        }
        catch (Exception exc) {
			log(exc.toString());
			throw new ServletException(exc);
        }
	}
}
